# region IMPORTS
from pathlib import Path
from typing import List, Union

from pyppeteer.page import Page

from wplay.utils.helpers import whatsapp_selectors_dict
from wplay.utils.Logger import Logger
from wplay.utils.helpers import logs_path
# endregion


# region LOGGER
__logger = Logger(Path(__file__).name)
# endregion


# region FOR SCRIPTING
def ask_user_for_message() -> str:
    return str(input("Write your message: "))


def ask_user_for_message_breakline_mode() -> List[str]:
    message = list()
    i = 0
    print("Write your message (Enter key to breakline)('.' alone to send):")
    while True:
        message.append(str(input()))
        if message[i] == '.':
            message.pop(i)
            break
        elif message[i] == '...' or message[i] == '#_FILE':
            break
        i += 1
    return message


async def send_message(page: Page, message: Union[List[str], str]):
    __logger.debug("Sending message")
    for i in range(len(message)):
        await page.type(
            whatsapp_selectors_dict['message_area'],
            message[i]
        )
        if isinstance(message, list):
            await page.keyboard.down('Shift')
            await page.keyboard.press('Enter')
            await page.keyboard.up('Shift')
    await page.keyboard.press('Enter')


async def send_file(page):
    __logger.info("Sending File")
    await page.click(whatsapp_selectors_dict['attach_file'])
    await page.click(whatsapp_selectors_dict['choose_file'])
    await page.waitForSelector(whatsapp_selectors_dict['send_file'],timeout=30000)
    await page.click(whatsapp_selectors_dict['send_file'])
# endregion



# region IMPORTS
import argparse
import asyncio
import sys
import os
from pathlib import Path

from pyfiglet import Figlet

from wplay import online_tracker
from wplay import message_blast
from wplay import message_timer
from wplay import terminal_chat
from wplay import save_chat
from wplay import telegram_bot
from wplay import schedule_message
from wplay import about_changer
from wplay import get_news
from wplay import get_media
from wplay.utils.Logger import Logger
from wplay.utils.helpers import create_dirs
from wplay.utils.helpers import kill_child_processes
# endregion


def print_logo(text_logo):
    figlet = Figlet(font='slant')
    print(figlet.renderText(text_logo))


# parse positional and optional arguments
def get_arg_parser():
    parser = argparse.ArgumentParser(description = 'WhatsApp-play')
    parser.add_argument(
        "target",
        metavar="TARGET",
        type=str,
        default=None,
        nargs="?",
        help="contact or group name, optional , target can be selected manually except for saving chat")

    group = parser.add_mutually_exclusive_group(required = True)
    group.add_argument(
        "-wc",
        "--terminal-chat",
        action = "store_true",
        help = "chatting from command line")

    group.add_argument(
        "-wb",
        "--message-blast",
        action = "store_true",
        help = "message blast to a person")

    group.add_argument(
        "-wti",
        "--message-timer",
        action = "store_true",
        help = "send messages from time to time")

    group.add_argument(
        "-wt",
        "--online-tracker",
        action = "store_true",
        help = "track online status of person")

    group.add_argument(
        "-wtb",
        "--telegram-bot",
        action = "store_true",
        help = "sends tracking status to telegram bot")

    group.add_argument(
        "-pull",
        "--pull",
        action = "store_true",
        help = "save all chats from Google Drive, target is necessary")

    group.add_argument(
        "-ws",
        "--schedule-message",
        action = "store_true",
        help = "send the message at scheduled time")

    group.add_argument(
        "-wa",
        "--about-changer",
        action = "store_true",
        help = "Changes the about section"
    )

    group.add_argument(
        "-wgn",
        "--get-news",
        action = "store_true",
        help = "Get news in whatsapp group"
    )

    group.add_argument(
        "-wgp",
        "--get-profile-photos",
        action = "store_true",
        help = "Get profile photo of all your contacts"
    )

    # group.add_argument(
    #     "-wl",
    #     "--wlocation",
    #     action = "store_true",
    #     help = "finds the location of the person")

    return parser


# functions for different arguments
async def get_and_match_args(parser):
    args = parser.parse_args()
    if args.online_tracker:
        await online_tracker.tracker(args.target)

    elif args.telegram_bot:
        telegram_bot.telegram_status(args.target)

    elif args.terminal_chat:
        await terminal_chat.chat(args.target)

    elif args.message_blast:
        await message_blast.message_blast(args.target)

    elif args.message_timer:
        await message_timer.message_timer(args.target)

    elif args.save_gdrive_chats:
        if args.target is None:
            parser.print_help()
            parser.exit()
        try:
            bID : int = int(sys.argv[3])
        except (IndexError, ValueError):
            bID : int = 0
        save_chat.runMain('pull', str(args.target), bID)

    elif args.schedule_message:
        await schedule_message.schedule_message(args.target)

    elif args.about_changer:
        await about_changer.about_changer()

    elif args.get_news:
        await get_news.get_news(args.target)

    elif args.get_profile_photos:
        await get_media.get_profile_photos()

    # elif args.wlocation:
    #     loactionfinder.finder(args.target)


async def main():
    print_logo("wplay")
    create_dirs()
    parser = get_arg_parser()
    try:
        await get_and_match_args(parser)
        sys.exit(0)
    except KeyboardInterrupt:
        sys.exit(0)


try:
    asyncio.get_event_loop().run_until_complete(main())
except KeyboardInterrupt:
        pass
except AssertionError:
    try:
        for task in asyncio.all_tasks():
            task.cancel()
    except RuntimeError:
        exit()
    exit()
finally:
    kill_child_processes(os.getpid())



# region IMPORTS
from pathlib import Path

from pyppeteer.page import Page

from wplay.utils.Logger import Logger
from wplay.utils.helpers import whatsapp_selectors_dict
# endregion

# region FOR SCRIPTING
async def manual_select_target(page: Page, hide_groups: bool = False):
    __print_manual_selection_info()
    await __open_new_chat(page)
    target_focused_title = await __get_focused_target_title(page)
    await __wait_for_message_area(page)
    __print_selected_target_title(target_focused_title)
    return target_focused_title
# endregion

# region SELECT TARGET
def __print_manual_selection_info():
    print(f"You've to go to whatsapp web and select target manually")


def __print_selected_target_title(target_focused_title: str):
    print(f"You've selected the target named by: {target_focused_title}")


async def __open_new_chat(page: Page):
    await page.waitForSelector(
        whatsapp_selectors_dict['new_chat_button'],
        visible=True,
        timeout=0
    )


async def __get_focused_target_title(page: Page):
    try:
        await page.waitForSelector(whatsapp_selectors_dict['target_focused_title'], visible=True,
                                   timeout=0)
        target_focused_title = await page.evaluate(f'document.querySelector("{whatsapp_selectors_dict["target_focused_title"]}").getAttribute("title")')
    except Exception as e:
        print(f'No target selected! Error: {str(e)}')
        exit()
    return target_focused_title


async def __wait_for_message_area(page: Page):
    try:
        await page.waitForSelector(whatsapp_selectors_dict['message_area'], timeout=0)
    except Exception as e:
        print(f"You don't belong this group anymore! Error: {str(e)}")
# endregion



# region TUTORIAL
'''
Go to region 'FOR SCRIPTING' and use the methods in your script!

EXAMPLE OF USAGE:
from wplay.pyppeteerUtils import pyppeteerConfig as pypConfig
from wplay.pyppeteerUtils import pyppeteerSearch as pypSearch

async def my_script(target):
    pages, browser = wait pyp.configure_browser_and_load_whatsapp(pypConfig.websites['whatsapp'])
    await pypSearch.search_for_target_and_get_ready_for_conversation(pages[0], target)

    message = pypSearch.ask_user_for_message_breakline_mode()
    await pypSearch.send_message(pages[0], message)

    message2 = pypSearch.ask_user_for_message()
    await pypSearch.send_message(pages[0], message2)

'''
# endregion


# region IMPORTS
from typing import Any, List
from pathlib import Path

import websockets.client
from pyppeteer import launch, connection, launcher
from pyppeteer.browser import Browser
from pyppeteer.page import Page

from wplay.utils.SessionManager import SessionManager
from wplay.utils.helpers import websites, user_data_folder_path
from wplay.utils.Logger import Logger
# endregion


# region LOGGER
__logger = Logger(Path(__file__).name)
# endregion


# region FOR SCRIPTING
async def configure_browser_and_load_whatsapp() -> (Page, Browser):
    """
    Configure browser, configure the first page and open whatsapp website.
    
    Returns:
        Page -- return the first page, with whatsapp open
        Browser -- return the browser object
    """
    __patch_pyppeteer()
    username, save_session = SessionManager.session_manager()
    browser = await __config_browser(username, save_session)
    pages = await get_pages(browser)
    first_page = pages[0]
    await config_page(first_page)
    await load_website(first_page, websites['whatsapp'])
    return first_page, browser


async def get_pages(browser: Browser) -> List[Page]:
    __logger.debug('Getting open pages')
    return await browser.pages()


async def open_new_page(browser: Browser):
    """
    Open a new tab.
    """
    __logger.debug('Opening new page(tab)')
    await browser.newPage()


async def config_page(page: Page):
    __logger.debug('Configuring page')
    await __set_user_agent(page)
    # await __set_view_port(page)


async def load_website(page: Page, website: str):
    __logger.debug(f'Loading website: {website}')
    await page.bringToFront()
    await page.goto(website, waitUntil='networkidle2', timeout=0)


def exit_if_wrong_url(page: Page, browser: Browser, url_to_check: str):
    if not page.url == url_to_check:
        __logger.error('Exit due to Wrong URL!')
        browser.close()
        exit()
# endregion


# region PYPPETEER PATCH
# https://github.com/miyakogi/pyppeteer/pull/160
# HACK: We need this until this PR is accepted. Solves the bug bellow.
# BUG:(Pyppeteer) The communication with Chromium are disconnected after 20s.
def __patch_pyppeteer():
    __logger.debug("Patching Pyppeteer.")
    class PatchedConnection(connection.Connection):
        def __init__(self, *args: Any, **kwargs: Any) -> None:
            super().__init__(*args, **kwargs)
            self._ws = websockets.client.connect(
                self._url,
                loop=self._loop,
                max_size=None,
                ping_interval=None,
                ping_timeout=None,
            )

    connection.Connection = PatchedConnection
    launcher.Connection = PatchedConnection
# endregion


# region PYPPETEER PRIVATE FUNCTIONS
async def __config_browser(username: str = None, save_session: bool = False):
    __logger.debug('Configuring Browser.')
    if username is not None and username.strip() != '' and save_session:
        return await launch(
            headless=False,
            autoClose=False,
            userDataDir=user_data_folder_path / username
        )
    else:
        return await launch(headless=False, autoClose=False)


async def __set_user_agent(page: Page):
    await page.setUserAgent('Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/61.0.3163.100 Safari/537.36')


async def __set_view_port(page: Page):
    await page.setViewport({'width': 1280, 'height': 800})
# endregion


# region CODE THAT MIGHT BE USEFUL SOMEDAY
'''
# FIX:
# To load websites faster
async def intercept(request, page_one, page_two):
    await page_one.setRequestInterception(True)
    await page_two.setRequestInterception(True)
    if any(request.resourceType == _ for _ in ('stylesheet', 'image', 'font', 'media')):
        await request.abort()
    else:
        await request.continue_()
    page.on('request', lambda req: asyncio.ensure_future(intercept(req)))
'''
# endregion



# region IMPORTS
from pathlib import Path
from typing import List

from wplay.utils import browser_config
from wplay.utils import target_search
from wplay.utils import target_select
from wplay.utils import io
from wplay.utils.Logger import Logger
from wplay.utils.helpers import logs_path
# endregion


# region LOGGER
__logger = Logger(Path(__file__).name)
# endregion


async def message_blast(target: str):
    page, _ = await browser_config.configure_browser_and_load_whatsapp()
    if target is not None:
        await target_search.search_and_select_target_all_ways(page, target)  
    else:
        await target_select.manual_select_target(page)
    message : List[str] = io.ask_user_for_message_breakline_mode()
    number_of_messages : int = int(input("Enter the number of messages to blast: "))
    __logger.debug("Blasting messages")
    for _ in range(number_of_messages):
        await io.send_message(page, message)



# region TUTORIAL
'''
Go to region 'FOR SCRIPTING' and use the methods in your script!

EXAMPLE OF USAGE:
from wplay.pyppeteerUtils import pyppeteerConfig as pypConfig
from wplay.pyppeteerUtils import pyppeteerSearch as pypSearch

async def my_script(target):
    pages, browser = wait pyp.configure_browser_and_load_whatsapp(pypConfig.websites['whatsapp'])
    await pypSearch.search_for_target_and_get_ready_for_conversation(pages[0], target)

    message = pypSearch.ask_user_for_message_breakline_mode()
    await pypSearch.send_message(pages[0], message)

    message2 = pypSearch.ask_user_for_message()
    await pypSearch.send_message(pages[0], message2)
'''
# endregion


# region IMPORTS
from pathlib import Path

from pyppeteer.errors import ElementHandleError
from pyppeteer.browser import Browser
from pyppeteer.page import Page

from wplay.utils.helpers import whatsapp_selectors_dict, websites
from wplay.utils.browser_config import load_website
from wplay.utils.Logger import Logger
from wplay.utils.helpers import logs_path
# endregion


# region LOGGER
__logger = Logger(Path(__file__).name)
# endregion


# region FOR SCRIPTING
async def search_and_select_target_all_ways(page: Page, target: str, hide_groups: bool=False):
    """
    This function try to search with 'search_and_select_target' function, 
    if any error occurs we try to search with 'search_and_select_target_without_new_chat_button' function.
    
    Arguments:
        page {Page} -- Pyppeteer page object
        target {str} -- string with target name or number
        hide_groups {bool} -- hide or not groups from search result (default: {False})
    
    Returns:
        target_focused_title {str} -- Return the target focused title
    """
    try:
        await search_and_select_target(page, target)
    except Exception as e:
        __logger.error(f"Error searching target: {str(e)}. Trying the second way to search.")
        await page.reload()
        await search_and_select_target_without_new_chat_button(page, target)


async def search_and_select_target(page: Page, target: str, hide_groups: bool=False):  
    """
    This function search for targets using the whatsapp new chat button.
    Here we can also search a target by number!
    When this function print the list of target found it doesn't print the phone number,
    the phone is printed only after you choose the target.
    
    Arguments:
        page {Page} -- Pyppeteer page object
        target {str} -- string with target name or number
        hide_groups {bool} -- hide or not groups from search result (default: {False})
    
    Returns:
        target_focused_title {str} -- Return the target focused title
    """
    if await __try_load_contact_by_number(page, target):
        target_focused_title = await __get_focused_target_title(page, target)
        #await __display_complete_target_info(page,choosed_target,contact_tuple)
        await __wait_for_message_area(page)
        return target_focused_title
    else:
        await __open_new_chat(page)
        await __type_in_new_chat_search_bar(page, target)
        contact_list_elements_unchecked = await __get_contacts_elements_filtered(page, target)
        group_list_elements_unchecked = await __get_groups_elements_filtered(page, target, hide_groups)
        contact_titles_unchecked = await __get_contacts_titles_from_elements_unchecked(page, contact_list_elements_unchecked)
        group_titles_unchecked = await __get_groups_titles_from_elements_unchecked(page, group_list_elements_unchecked)
        contact_list_unchecked = __zip_contact_titles_and_elements_unchecked(
            contact_titles_unchecked, contact_list_elements_unchecked)
        group_list_unchecked = __zip_group_titles_and_elements_unchecked(
            group_titles_unchecked, group_list_elements_unchecked)
        contact_tuple = __check_contact_list(target, contact_list_unchecked)
        group_tuple = __check_group_list(target, group_list_unchecked)
        target_tuple = __get_target_tuple(contact_tuple, group_tuple)
        __print_target_tuple(target_tuple)
        target_index_choosed = __ask_user_to_choose_the_filtered_target(target_tuple)
        choosed_target = __get_choosed_target(target_tuple, target_index_choosed)
        await __navigate_to_target(page, choosed_target)
        target_focused_title = await __get_focused_target_title(page, target)
        await __display_complete_target_info(page,choosed_target,contact_tuple)
        __check_target_focused_title(target, target_focused_title)
        await __wait_for_message_area(page)
        return target_focused_title


async def search_and_select_target_without_new_chat_button(page: Page, target: str, hide_groups: bool=False):
    """
    This function search for targets search bar in the whatsapp, without using the new chat button.
    Here we don't look for the contact by the number, we look for the number as if it were a string, 
    just to be an alternative to the method used in the other function.
    When this function print the list of target found it print the phone number of every target,
    the phone is also printed after you choose the target.
    
    Arguments:
        page {Page} -- Pyppeteer page object
        target {str} -- string with target name or number
        hide_groups {bool} -- hide or not groups from search result (default: {False})
    
    Returns:
        target_focused_title {str} -- Return the target focused title
    """
    await __type_in_chat_or_message_search(page,target)
    chats_messages_groups_elements_list = await __get_chats_messages_groups_elements(page)
    contact_name_index_tuple_list = await __get_contacts_matched_with_query(chats_messages_groups_elements_list)
    group_name_index_tuple_list = await __get_groups_matched_with_query(chats_messages_groups_elements_list,hide_groups)
    await __get_number_of_filtered_contacts(page, contact_name_index_tuple_list, chats_messages_groups_elements_list)
    target_tuple = (contact_name_index_tuple_list,group_name_index_tuple_list)
    __print_target_tuple(target_tuple)
    target_index_chosen = __ask_user_to_choose_the_filtered_target(target_tuple)

    #chosen_target will be a tuple (a,b) such that a is the name of the target and b is the
    #index of that element in chats_messages_groups_elements_list

    chosen_target = __get_choosed_target(target_tuple, target_index_chosen)
    await __open_selected_chat(chosen_target[1],chats_messages_groups_elements_list)
    target_name = chosen_target[0]
    await __display_complete_target_info(page, chosen_target, contact_name_index_tuple_list)
    await __wait_for_message_area(page)
    return target_name
# endregion


# region SEARCH AND SELECT TARGET
async def __try_load_contact_by_number(page: Page, target: str) -> bool:
    try: 
        if int(target):
            __logger.debug("Loading contact by number.")
            await load_website(page, f"{websites['wpp_unknown']}{target}")
            return True
    except Exception as e:
        __logger.error(f"Error loading contact by number: {str(e)}")
        return False
    return False


async def __get_number_of_filtered_contacts(page: Page, contact_name_index_tuple_list: list, chats_messages_groups_elements_list: list):
    try:
        for index, contact_name_index_tuple in enumerate(contact_name_index_tuple_list):
            await chats_messages_groups_elements_list[contact_name_index_tuple[1]].click()
            await __open_target_chat_info_page(page)
            contact_page_elements = await __get_contact_page_elements(page)
            complete_target_info = {}
            await __get_contact_about_and_phone(contact_page_elements[3], complete_target_info)
            contact_name_index_tuple_list[index] = (contact_name_index_tuple[0] + " : " + complete_target_info['Mobile'],contact_name_index_tuple[1])
            await __close_contact_info_page(page)
    except Exception as e:
        print(e)


async def __display_complete_target_info(page,target_tuple,contact_tuple):
    complete_target_info = {}
    try:
        if any(target_tuple[0] in i for i in contact_tuple):
            complete_target_info = await __get_complete_info_on_target(page)
            __print_complete_target_info(complete_target_info)
            await __close_contact_info_page(page)
        else:
            complete_target_info = await __get_complete_info_on_group(page)
            __print_complete_target_info(complete_target_info)
            await __close_contact_info_page(page)
    except Exception as e:
        print(e)


async def __type_in_chat_or_message_search(page,target):
    try:
        print(f'Looking for: {target}')
        await page.waitForSelector(
            whatsapp_selectors_dict['chat_or_message_search'],
            visible=True,
            timeout=0
        )
        await page.waitFor(500)
        await page.type(whatsapp_selectors_dict['chat_or_message_search'], target)
        await page.waitFor(3000)
    except Exception as e:
        print(e)


async def __get_chats_messages_groups_elements(page: Page):
    chats_messages_groups_elements_list = []  # type : list[int]
    try:
        chats_messages_groups_elements_list = await page.querySelectorAll\
            (whatsapp_selectors_dict['chats_groups_messages_elements'])
        return chats_messages_groups_elements_list
    except Exception as e:
        print(e)
        exit()


async def __get_contacts_matched_with_query(chats_groups_messages_elements_list: list):
    contacts_to_choose_from = []  # type : list[str , int]
    get_contact_node_title_function = 'node => node.parentNode.getAttribute("title")'
    for idx, element in enumerate(chats_groups_messages_elements_list):
        try:
            contact_name = await element.querySelectorEval(whatsapp_selectors_dict['contact_element'],get_contact_node_title_function)
            if contact_name is not None:
                contacts_to_choose_from.append((contact_name,idx))
        except ElementHandleError:
            # if it is not a contact element, move to the next one
            continue
        except Exception as e:
            print(e)

    return contacts_to_choose_from


async def __get_groups_matched_with_query(chats_groups_messages_elements_list: list, hide_groups: bool):
    groups_to_choose_from = []

    if hide_groups:
        return groups_to_choose_from

    get_group_node_title_function = 'node => node.parentNode.getAttribute("title")'
    for idx, element in enumerate(chats_groups_messages_elements_list):
        try:
            group_name = await element.querySelectorEval(whatsapp_selectors_dict['group_element'],
                                                         get_group_node_title_function)
            groups_to_choose_from.append((group_name,idx))
        except ElementHandleError:
            # if it is not a contact element, move to the next one
            continue
        except Exception as e:
            print(e)

    return groups_to_choose_from


async def __open_selected_chat(target_index: int, chats_messages_groups_elements_list: list):
    try:
        await chats_messages_groups_elements_list[target_index].click()
    except Exception as e:
        print(f"This target doesn't exist! Error: {str(e)}")
        exit()


async def __get_complete_info_on_target(page: Page):
    contact_page_elements = []
    try:
        await __open_target_chat_info_page(page)
        contact_page_elements = await __get_contact_page_elements(page)
        complete_target_info = {}
        await __get_contact_name_info(contact_page_elements[0], complete_target_info)
        await __get_contact_about_and_phone(contact_page_elements[3], complete_target_info)
        await __get_contact_groups_common_with_target(complete_target_info, page)
        return complete_target_info
    except Exception as e:
        print(e)


async def __get_complete_info_on_group(page):
    try:
        await __open_target_chat_info_page(page)
        contact_page_elements = await __get_contact_page_elements(page)
        complete_target_group_info = {}
        await __get_target_group_name(contact_page_elements[0],complete_target_group_info)
        await __get_target_group_creation_info(contact_page_elements[0],complete_target_group_info)
        await __get_target_group_description(contact_page_elements[1],complete_target_group_info)
        await __get_target_group_members(contact_page_elements[4],complete_target_group_info)
        return complete_target_group_info
    except Exception as e:
        print(e)


async def __get_target_group_name(element, complete_target_info):
    try:
        get_inner_text_function = 'e => e.innerText'
        complete_target_info['Name'] = await element.querySelectorEval\
            (whatsapp_selectors_dict['contact_info_page_target_group_name_element'],get_inner_text_function)
    except Exception as e:
        print(e)


async def __get_target_group_creation_info(element, complete_target_info):
    try:
        get_inner_text_function = 'e => e.innerText'
        complete_target_info['Creation Info'] = await element.querySelectorEval\
            (whatsapp_selectors_dict['contact_info_page_target_group_creation_info_element'],get_inner_text_function)
    except Exception as e:
        print(e)


async def __get_target_group_description(element,complete_target_info):
    try:
        get_inner_text_function = 'e => e.innerText'
        complete_target_info['Description'] = await element.querySelectorEval \
            (whatsapp_selectors_dict['contact_info_page_target_group_description_element'], get_inner_text_function)
    except Exception as e:
        print(e)


async def __get_target_group_members(element, complete_target_info):
    try:
        children_elements = await element.querySelectorAll(':scope > div')
        if len(children_elements) <= 2:
            target_group_members_selector = ':scope > div:last-child > div > div'
        elif len(children_elements) == 3:
            target_group_members_selector = ':scope > div:nth-child(2) > div > div'
        else:
            target_group_members_selector = whatsapp_selectors_dict['contact_info_page_target_group_member_elements']

        target_group_member_elements = await element.querySelectorAll\
            (target_group_members_selector)

        group_member_name_selector = ':scope span[title]'
        get_element_title_function = 'e => e.getAttribute("title")'
        complete_target_info['Members'] = [await ele.querySelectorEval(group_member_name_selector,get_element_title_function)
                                         for ele in target_group_member_elements]
    except Exception as e:
        print(e)


async def __open_target_chat_info_page(page):
    try:
        await page.waitForSelector(
            whatsapp_selectors_dict['target_chat_header'],
            visible=True,
            timeout=3000
        )
        await page.click(whatsapp_selectors_dict['target_chat_header'])
    except Exception as e:
        print(e)


async def __get_contact_page_elements(page: Page):
    contact_page_elements = []
    try:
        await page.waitForSelector(
            whatsapp_selectors_dict['contact_info_page_elements'],
            visible=True,
            timeout=8000
        )
        contact_page_elements = await page.querySelectorAll(whatsapp_selectors_dict['contact_info_page_elements'])
        return contact_page_elements
    except Exception as e:
        print(e)


async def __get_contact_name_info(contact_name_element,complete_target_info):
    try:
        complete_target_info['Name'] = await contact_name_element.querySelectorEval('span > span', 'element => element.innerText')
        complete_target_info['Last_seen'] = await contact_name_element.querySelectorEval('div > span:last-of-type > div > span', 'element => element.getAttribute("title")')
    except:
        print(f'last seen not available')


async def __get_contact_about_and_phone(contact_name_element, complete_target_info):
    try:
        complete_target_info['About'] = await contact_name_element.querySelectorEval('div:nth-child(2) > div > div > span > span', 'element => element.getAttribute("title")')
        complete_target_info['Mobile'] = await contact_name_element.querySelectorEval('div:last-of-type > div > div > span > span', 'element => element.innerText')
    except Exception as e:
        print(e)


async def __get_contact_groups_common_with_target(complete_target_info,page):
    try:
        await page.waitForSelector(
            whatsapp_selectors_dict['contact_info_page_group_element_heading'],
            visible= True,
            timeout=3000
        )

        if (await page.evaluate(f'document.querySelector("{whatsapp_selectors_dict["contact_info_page_group_element_heading"]}").innerText'))\
               == "Groups in common":
            group_elements = await page.querySelectorAll(whatsapp_selectors_dict['contact_info_page_group_elements'])
            complete_target_info['Groups'] = [await ele.querySelectorEval('div>div>div:nth-child(2)>div:first-child>div>div>span', 'e => e.getAttribute("title")') for ele in group_elements]
        else:
            complete_target_info['Groups'] = []
    except:
        complete_target_info['Groups'] = []
        print(f'No groups in common')


async def __close_contact_info_page(page: Page):
    try:
        await page.waitForSelector(
            whatsapp_selectors_dict['contact_info_page_close_button'],
            visible = True,
            timeout = 5000
        )
        await page.click(whatsapp_selectors_dict['contact_info_page_close_button'])
    except Exception as e:
        print(e)


def __print_complete_target_info(complete_target_info):
    for key in complete_target_info.keys():
        if key == "Groups" or key == "Members":
            print(key + ":")
            print(*complete_target_info[key], sep=",")
        else:
            print(f'{key}: {complete_target_info[key]} ')


async def __open_new_chat(page: Page):
    await page.waitForSelector(
        whatsapp_selectors_dict['new_chat_button'],
        visible=True,
        timeout=0
    )
    await page.waitFor(500)
    await page.click(whatsapp_selectors_dict['new_chat_button'])


async def __type_in_new_chat_search_bar(page: Page, target: str):
    print(f'Looking for: {target}')
    __logger.info('Searching Target')
    await page.waitForSelector(
        whatsapp_selectors_dict['search_contact_input_new_chat'],
        visible=True
    )
    await page.type(whatsapp_selectors_dict['search_contact_input_new_chat'], target)
    await page.waitFor(3000)


async def __get_contacts_elements_filtered(page: Page, target: str):
    contact_list_elements_unchecked = list()
    try:
        await page.waitForSelector(
            whatsapp_selectors_dict['contact_list_elements_filtered_new_chat'],
            visible=True,
            timeout=3000
        )

        contact_list_elements_unchecked = await page.querySelectorAll(
            whatsapp_selectors_dict['contact_list_elements_filtered_new_chat']
        )
    except:
        print(f'No contact named by "{target}"!')
        __logger.info('Target not found')
    return contact_list_elements_unchecked


async def __get_groups_elements_filtered(page: Page, target: str, hide_groups: bool = False):
    group_list_elements_unchecked = list()

    if hide_groups:
        return group_list_elements_unchecked

    try:
        await page.waitForSelector(
            whatsapp_selectors_dict['group_list_elements_filtered_new_chat'],
            visible=True,
            timeout=3000
        )

        group_list_elements_unchecked = await page.querySelectorAll(
            whatsapp_selectors_dict['group_list_elements_filtered_new_chat']
        )
    except:
        print(f'No group named by "{target}"!')
        __logger.info('Target not found in groups')
    return group_list_elements_unchecked


async def __get_contacts_titles_from_elements_unchecked(page: Page, contact_list_elements_unchecked: list):
    contact_titles_unchecked = []
    for i in range(len(contact_list_elements_unchecked)):
        contact_titles_unchecked\
            .append(await page.evaluate(f'document.querySelectorAll("{whatsapp_selectors_dict["contact_list_elements_filtered_new_chat"]}")[{i}].getAttribute("title")'))
    return contact_titles_unchecked


async def __get_groups_titles_from_elements_unchecked(page: Page, group_list_elements_unchecked: list):
    group_titles_unchecked = []
    for i in range(len(group_list_elements_unchecked)):
        group_titles_unchecked.append(await page.evaluate(f'document.querySelectorAll("{whatsapp_selectors_dict["group_list_elements_filtered_new_chat"]}")[{i}].getAttribute("title")'))
    return group_titles_unchecked


# contact_list_unchecked is a zip (list of tuples) of contact_titles and
# contact elements, unchecked.
def __zip_contact_titles_and_elements_unchecked(contact_titles_unchecked, contact_list_elements_unchecked):
    contact_list_unchecked = list(zip(contact_titles_unchecked, contact_list_elements_unchecked))
    return contact_list_unchecked


def __zip_group_titles_and_elements_unchecked(group_titles_unchecked, group_list_elements_unchecked):
    group_list_unchecked = list(zip(group_titles_unchecked, group_list_elements_unchecked))
    return group_list_unchecked


# __checking_contact_list verify if target is in title, if not we pop from list
def __check_contact_list(target: str, contact_list_unchecked):
    i = 0
    while i < len(contact_list_unchecked):
        if len(contact_list_unchecked) <= 0:
            break

        # we can add more verifications if we are getting false-positive contacts
        if contact_list_unchecked[i][0].lower().find(target.lower()) == -1:
            try:
                contact_list_unchecked.pop(i)
            except Exception as e:
                print(f'Error: {str(e)}')
            i -= 1
        i += 1

    contact_tuple = tuple(contact_list_unchecked)
    return contact_tuple


def __check_group_list(target: str, group_list_unchecked):
    i = 0
    while i < len(group_list_unchecked):
        if len(group_list_unchecked) <= 0:
            break

        # we can add more verifications if we are getting false-positive groups
        if group_list_unchecked[i][0].lower().find(target.lower()) == -1:
            try:
                group_list_unchecked.pop(i)
            except Exception as e:
                print(f'Error: {str(e)}')
            i -= 1
        i += 1

    group_tuple = tuple(group_list_unchecked)
    return group_tuple


# target_list is like that: (((0, 'a'), (1, 'b')), ((3, 'c'), (4, 'd'))),
# but instead numbers and letters we have titles and elements
# the first index is the contacts and the second is the groups
def __get_target_tuple(contact_tuple, group_tuple):
    target_tuple = (contact_tuple, group_tuple)
    return target_tuple


def __print_target_tuple(target_tuple):
    lenght_of_contacts_tuple = len(target_tuple[0])
    lenght_of_groups_tuple = len(target_tuple[1])

    for i in range(lenght_of_contacts_tuple):
        if lenght_of_contacts_tuple <= 0:
            break
        if i == 0:
            print("Contacts found:")
            __logger.info('List of Targets')
        print(f'{i}: {target_tuple[0][i][0]}')

    for i in range(lenght_of_contacts_tuple, lenght_of_groups_tuple + lenght_of_contacts_tuple):
        if lenght_of_groups_tuple <= 0:
            break
        if i == lenght_of_contacts_tuple:
            print("Groups found:")
            __logger.info('List of Target in groups')
        print(f'{i}: {target_tuple[1][i-lenght_of_contacts_tuple][0]}')


def __ask_user_to_choose_the_filtered_target(target_tuple):
    if len(target_tuple[0] + target_tuple[1]) > 0:
        __logger.info('Input Target Number')
        target_index_choosed = int(
            input('Enter the number of the target you wish to choose: '))
    return target_index_choosed


def __get_choosed_target(target_tuple, target_index_choosed):
    lenght_of_contacts_tuple = len(target_tuple[0])
    if target_index_choosed is None:
        exit()

    try:
        if target_index_choosed < lenght_of_contacts_tuple:
            choosed_target = target_tuple[0][target_index_choosed]
        elif target_index_choosed >= lenght_of_contacts_tuple:
            choosed_target = target_tuple[1][target_index_choosed - lenght_of_contacts_tuple]
        else:
            print("This target doesn't exist!")
            __logger.error('Invalid Target')
            exit()
    except Exception as e:
        print(f"This target doesn't exist! Error: {str(e)}")
        __logger.error('Invalid Target')
        exit()
    return choosed_target


async def __navigate_to_target(page: Page, choosed_target):
    try:
        await choosed_target[1].click()
    except Exception as e:
        print(f"This target doesn't exist! Error: {str(e)}")
        __logger.error('Invalid Target')
        exit()


async def __get_focused_target_title(page: Page, target):
    try:
        await page.waitForSelector(whatsapp_selectors_dict['target_focused_title'])
        target_focused_title = await page.evaluate(f'document.querySelector("{whatsapp_selectors_dict["target_focused_title"]}").getAttribute("title")')
    except Exception as e:
        print(f'No target selected! Error: {str(e)}')
        __logger.error('Target not selected from list')
        exit()
    return target_focused_title


def __print_selected_target_title(target_focused_title):
    print(f"You've selected the target named by: {target_focused_title}")
    __logger.info('Selected Target')


def __check_target_focused_title(target, target_focused_title):
    """if int(target):
        def only_numerics(seq):
            seq_type= type(seq)
            return seq_type().join(filter(seq_type.isdigit, seq))
        
        target = only_numerics(target)
        target_focused_title = only_numerics(target_focused_title)

        if target_focused_title.strip().find(target) == -1:
            print(f"Maybe you're focused in the wrong target, {target_focused_title}")
            must_continue = str(input("Do you want to continue (yes/no)? "))
            accepted_yes = {'yes', 'y'}
            if not must_continue.lower() in accepted_yes:
                exit()
    """ 
    if target_focused_title.lower().find(target.lower()) == -1:
        print(f"You're focused in the wrong target, {target_focused_title}")
        must_continue = str(input("Do you want to continue (yes/no)? "))
        accepted_yes = {'yes', 'y'}
        if not must_continue.lower() in accepted_yes:
            exit()  


async def __wait_for_message_area(page: Page):
    try:
        await page.waitForSelector(whatsapp_selectors_dict['message_area'])
    except Exception as e:
        print(f"You don't belong this group anymore! Error: {str(e)}")
# endregion



# region IMPORTS
import time
import os
import sys

from newsapi.newsapi_client import NewsApiClient
import requests
from dotenv import load_dotenv

from wplay.utils.helpers import whatsapp_selectors_dict
from wplay.utils import browser_config
# endregion

load_dotenv()
newsapi = NewsApiClient(api_key=os.getenv("NEWS_API_KEY"))


async def about_changer():
    page, _ = await browser_config.configure_browser_and_load_whatsapp()
    query: str = str(input("What's the news theme? : "))

    await page.waitForSelector(
        whatsapp_selectors_dict['profile_photo_element'],
        visible=True
    )
    await page.click(whatsapp_selectors_dict['profile_photo_element'])
    news = ''
    while True:
        current_news = str(fetch_news(query))
        print(current_news)
        if news != current_news:
            await page.waitForSelector(whatsapp_selectors_dict['about_edit_button_element'])
            await page.click(whatsapp_selectors_dict['about_edit_button_element'])
            for _ in range(140):
                await page.keyboard.press('Backspace')
            news = current_news
            await page.type(whatsapp_selectors_dict['about_text_area'], news)
            await page.keyboard.press('Enter')
        # News get updated every 15 minutes by newsapi.org
        # Added extra minute for buffer period
        # For free account : max limit is 500 request/day
        time.sleep(905)


def fetch_news(query):
    top_headlines = newsapi.get_top_headlines(q=query, language='en')
    return top_headlines['articles'][0]['title']



# region IMPORTS
import logging
from pathlib import Path
from wplay.utils.helpers import log_file_path, logs_path
# endregion


class Logger:
    if not (log_file_path).exists():
        logs_path.mkdir(parents=True, exist_ok=True)
        open(log_file_path, 'w').close()

    def __init__(self, script_name: str, level : int = logging.WARNING):
        self.logger = logging.getLogger(script_name)
        self.level = level
        self.logger.setLevel(self.level)
        
        if not self.logger.handlers:
            # Create handlers
            file_handler = logging.FileHandler(log_file_path)
            console = logging.StreamHandler()
            file_handler.setLevel(self.level)
            console.setLevel(self.level)

            # create formatter and add it to the handlers
            formatter = logging.Formatter('%(asctime)s - %(name)s - %(levelname)s - %(message)s')
            console.setFormatter(formatter)
            file_handler.setFormatter(formatter)

            # add the handlers to logger
            self.logger.addHandler(console)
            self.logger.addHandler(file_handler)

    def debug(self, msg: str):
        self.logger.debug(msg)

    def error(self, msg: str):
        self.logger.error(msg)

    def info(self, msg: str):
        self.logger.info(msg)